package com.xynet.marketing.utils;

import org.springframework.beans.BeansException;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.BeanDefinitionRegistry;
import org.springframework.beans.factory.support.DefaultListableBeanFactory;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.context.ConfigurableApplicationContext;
import org.springframework.stereotype.Component;

import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * spring bean工具
 * 修改bean只需要获取到bean对象再set修改属性即可
 */
@Component
public class SpringBeanUtils implements ApplicationContextAware {

    private static ConfigurableApplicationContext configurableApplicationContext;

    private static ApplicationContext applicationContext;

    @Override
    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        SpringBeanUtils.applicationContext = applicationContext;
        configurableApplicationContext = (ConfigurableApplicationContext) applicationContext;
    }

    /**
     * 获取bean
     */
    public static <T> T getBean(Class<T> clazz) {
        return applicationContext.getBean(clazz);
    }

    /**
     * 获取bean
     */
    public static Object getBean(String beanName) {
        return applicationContext.getBean(beanName);
    }

    /**
     * 获取bean
     */
    public static <T> T getBean(String beanName, Class<T> clazz) {
        return (T) applicationContext.getBean(beanName);
    }





    /**
     * 是否存在bean
     *
     * @param name beanName
     * @return
     */
    public static boolean existBean(String name) {
        return configurableApplicationContext.containsBean(name);
    }


    /**
     * 获取beanName
     */
    private static String getName(Class<?> clazz) {
        String result;
        String name = clazz.getName();
        result = name.substring(name.lastIndexOf(".") + 1);
        //首字母转小写
        result = result.substring(0, 1).toLowerCase() + result.substring(1);
        return result;
    }
}
