package com.xynet.marketing.utils.redis.utils;

import com.xynet.marketing.utils.FunctionUtils;
import com.xynet.marketing.utils.SpringBeanUtils;
import lombok.extern.slf4j.Slf4j;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;

import java.util.concurrent.TimeUnit;

@Slf4j
public class LockUtils {

    private String name;

    private boolean tryLock = false;

    private int timeout = 10;

    private int timelose = 10;

    private TimeUnit timeunit = TimeUnit.SECONDS;

    private FunctionUtils.NoParamsNoResult fialFunction;

    /**
     * 锁名称
     *
     * @param name
     * @return
     */
    public LockUtils name(String name) {
        this.name = name;
        return this;
    }

    /**
     * 开启非公平锁
     *
     * @return
     */
    public LockUtils tryLock(FunctionUtils.NoParamsNoResult fialFunction) {
        this.tryLock = true;
        this.fialFunction = fialFunction;
        return this;
    }

    /**
     * 开启非公平锁
     *
     * @param timeout 等待时间
     * @return
     */
    public LockUtils tryLock(int timeout, FunctionUtils.NoParamsNoResult fialFunction) {
        this.tryLock = true;
        this.timeout = timeout;
        this.fialFunction = fialFunction;
        return this;
    }

    /**
     * 开启非公平锁
     *
     * @param timeout 等待时间
     * @return
     */
    public LockUtils tryLock(int timeout) {
        this.tryLock = true;
        this.timeout = timeout;
        return this;
    }

    /**
     * 失效时间
     *
     * @param timelose
     * @return
     */
    public LockUtils timelose(int timelose) {
        this.timelose = timelose;
        return this;
    }

    /**
     * 时间单位
     *
     * @param timeunit
     * @return
     */
    public LockUtils timeunit(TimeUnit timeunit) {
        this.timeunit = timeunit;
        return this;
    }

    /**
     * 获取锁成功执行的函数
     *
     * @param successFunction
     * @param <T>
     * @return
     */
    public <T> T success(FunctionUtils.NoParamsResult<T> successFunction) {
        RedissonClient redissonClient = SpringBeanUtils.getBean(RedissonClient.class);
        RLock rlock = redissonClient.getLock(name);
        try {
            boolean fal;
            if (!tryLock) {
                rlock.lock(timelose, timeunit);
                fal = true;
            } else {
                fal = rlock.tryLock(timeout, timelose, timeunit);
            }
            if (fal) {
                return successFunction.run();
            }
            if (fialFunction != null) {
                fialFunction.run();
            }
        } catch (Exception e) {
            log.error("{}", e);
            throw new RuntimeException(e.getMessage());
        } finally {
            if (rlock.isLocked() && rlock.isHeldByCurrentThread()) {
                rlock.unlock();
            }
        }
        return null;
    }

    /**
     * 获取锁成功执行的函数
     *
     * @param successFunction
     * @return
     */
    public void success(FunctionUtils.NoParamsNoResult successFunction) {
        RedissonClient redissonClient = SpringBeanUtils.getBean(RedissonClient.class);
        RLock rlock = redissonClient.getLock(name);
        try {
            boolean fal;
            if (!tryLock) {
                rlock.lock(timelose, timeunit);
                fal = true;
            } else {
                fal = rlock.tryLock(timeout, timelose, timeunit);
            }
            if (fal) {
                successFunction.run();
                return;
            }
            if (fialFunction != null) {
                fialFunction.run();
            }
        } catch (Exception e) {
            log.error("{}", e);
            throw new RuntimeException(e.getMessage());
        } finally {
            if (rlock.isLocked() && rlock.isHeldByCurrentThread()) {
                rlock.unlock();
            }
        }
    }
}
